﻿// Project= SmartApp.IotDevice
// File=GeneralI2CDevice.cs
// Copyrights 2017 by Ronald Creutz, Harthaer Str, 18b, 01738 Dorfhain, Germany
// Visit me on http://www.ronald-creutz.de

using System.IO;
using System.Linq;

namespace SmartApp.IotDevice.Modules
{
    using System;
    using System.Collections.Generic;
    using System.Threading.Tasks;
    using Windows.Devices.Enumeration;
    using Windows.Devices.I2c;

    public class GeneralI2CDevice : IDisposable
    {
        #region members

        private const string BusId = "I2C1"; 
        private int _deviceId = 0;
        private I2cDevice _device = null;

        #endregion

        /// <summary>
        /// Initialize i2c device with given device id and try to connect to that device.
        /// </summary>
        /// <param name="deviceId">i2c device id.</param>
        /// <returns>Return initialization result.</returns>
        public async Task<bool> InitializeDeviceAsync(int deviceId)
        {
            _deviceId = deviceId;
            
            // get i2c bus devices
            string i2CDeviceSelector = I2cDevice.GetDeviceSelector(BusId); 
            IReadOnlyList<DeviceInformation> i2CDevices = await DeviceInformation.FindAllAsync(i2CDeviceSelector);
            if (i2CDevices.Count == 0)
            {
                return false;
            }

            var settings = new I2cConnectionSettings(_deviceId)
            {
                BusSpeed = I2cBusSpeed.StandardMode,
                SharingMode = I2cSharingMode.Shared
            };

            _device = await I2cDevice.FromIdAsync(i2CDevices[0].Id, settings);

            byte[] readBuf = { 0 };
            try
            {
                _device.Read(readBuf);
            }
            catch (Exception)
            {
                // todo: log ex
                return false;
            }

            // all was fine
            return true;
        }

        /// <summary>
        /// Sends data to i2c device.
        /// </summary>
        /// <param name="dataList">List of bytes.</param>
        /// <returns>Return given data.</returns>
        public List<byte> SetData(List<byte> dataList)
        {
            var oldData = dataList;

            try
            {
                // bus logic is negativ (0 = high, 1 = low), incoming data are 1 = high, 0 = low)
                _device.Write(dataList.Select(data => Convert.ToByte(0xFF - data)).ToArray());
            }
            catch (FileNotFoundException)
            {
                // sometime thrown (device logged off?)
            }

            return oldData;
        }

        /// <summary>
        /// Reads data from i2c device.
        /// </summary>
        /// <returns>Return data from device.</returns>
        public List<byte> ReadData()
        {
            List<byte> result = new List<byte>() { 0 };
            try
            {

                byte[] dataToRead = { 0 };
                _device.Read(dataToRead);

                // bus logic is negativ (0 = high, 1 = low), outgoing data are 1 = high, 0 = low)
                result = dataToRead.Select(data => Convert.ToByte(0xFF - data)).ToList();
            }
            catch (FileNotFoundException)
            {
                // sometime thrown (device logged off?)
            }

            return result;
        }

        public void Dispose()
        {
            _device?.Dispose();
        }
    }
}